package asd.td4;

import java.util.Vector;

/**
 * Une classe pour rsoudre le problme du placement de n reines libres
 * (c'est  dire qui ne sont pas en prise les unes avec les autres) sur un
 * chiquier de taille n. Pour un chiquier de taille n, soit de n X n cases
 * les lignes et les colonnes sont numrotes de 0  n - 1 et les reines 
 * sont numrotes de 0  n - 1, la reine numro i tant obligatoirement
 * dans la colonne numro i. On peut donc reprsent un chiquier de taille
 * n contenant k reines libres comme un k-uplet (L0,L1,..,Lk-1) avec
 * 0 <= k <= n. Une solution est un n-uplet (S0,S1,..,Sn-1) tel que pour tout
 * i, 0 <= i <= n-1, la reine i est ligne Si et en colonne i, et les n reines
 * sont libres. 
 */

public class Reines {

    private int n; // le nombre de reines a placer (n >= 1).
    private EchiquierDeReines edr; // l'chiquier commun

    public Reines(int nombreDeReines) {
        n = nombreDeReines;
    }

    /*
     * Retourne une solution, si elle existe, au problme des reines sur
     * un chiquier de taille n. <br>
     * Complexit : O(n^n).
    //
    public String uneSolution() {
	edr = new EchiquierDeReines(n);
	if ( uneSolution(0) )
	    return edr.toString();
	return "Pas de solutions";
    }

    /**
     * mthode rcursive associe  uneSolution() <br>
     * prcondition : 0 <= l < n <br>
     * l'chiquier contient j reines libres dans les j premires colonnes <br>
     * post-relation : s'il existe une solution globale avec les j premires
     * reines inchanges, la j+1 ime reine en ligne au moins l alors renvoie
     * vrai et edr contient cette solution <br>
     * sinon renvoie faux et edr est inchang. <br>
     */
    private boolean uneSolution(int l) {
	if ( edr.plein() )
	    return true;
	if ( ! edr.enPrise(l) ) {
	    edr.ajouterNouvelleReine(l);
	    if ( uneSolution(0) ) 
		return true;
	    edr.enleverDerniereReine();
	}
	if ( l == n-1 )
	    return false;
	return uneSolution(l+1);
    }

    /*
     * Retourne toutes les solutions au problme des reines sur
     * un chiquier de taille n.
     * Complexit : O(n^n).
     */
    public Vector toutesLesSolutions() {
        Vector v = new Vector();
	edr = new EchiquierDeReines(n);
	toutesLesSolutions(0, v);
	return v;
    }

    /* mthode rcursive associe  toutesLesSolutions()
     * prcondition : 0 <= l < n <br>
     * l'chiquier contient j reines libres dans les j premires colonnes <br>
     * post-relation : s'il existe des solutions globales avec les j premires
     * reines inchanges, la j+1 ime reine en ligne au moins l alors ces
     * solutions sont ajoutes  lesSolutions, <br>
     * edr est inchang. <br>
     */
    private void toutesLesSolutions(int l, Vector lesSolutions) {
	if ( edr.plein() )
	    lesSolutions.addElement(edr.toString());
	if ( ! edr.enPrise(l) ) {
	    edr.ajouterNouvelleReine(l);
	    toutesLesSolutions(0); 
	    edr.enleverDerniereReine();
	}
	if ( l < n-1 )
	    toutesLesSolutions(l+1);
    }
}

